// Copyright 2000-2022 JetBrains s.r.o. and contributors. Use of this source code is governed by the Apache 2.0 license.
package com.intellij.ide.customize.transferSettings.providers

import com.intellij.ide.IdeBundle
import com.intellij.ide.customize.transferSettings.models.Settings
import com.intellij.ide.plugins.PluginManagerCore
import com.intellij.openapi.progress.ProgressIndicator
import com.intellij.openapi.progress.Task
import com.intellij.openapi.project.Project
import com.intellij.util.application

open class TransferSettingsPerformImportTask(project: Project,
                                             private val performer: ImportPerformer,
                                             private var settings: Settings,
                                             private val shouldInstallPlugins: Boolean) : Task.Backgroundable(project,
                                                                                                              IdeBundle.message("transfersettings.task.progress.title.importing.settings"),
                                                                                                              false) {
  override fun run(indicator: ProgressIndicator) {
    indicator.isIndeterminate = true
    indicator.text2 = IdeBundle.message("transfersettings.task.progress.details.starting.up")
    val requiredPlugins = performer.collectAllRequiredPlugins(settings)
    indicator.isIndeterminate = false
    indicator.fraction = 0.0

    if (shouldInstallPlugins) {
      performer.installPlugins(project, requiredPlugins, indicator)
    }

    settings = performer.patchSettingsAfterPluginInstallation(settings, PluginManagerCore.getPlugins().map { it.pluginId.idString }.toSet())

    performer.perform(project, settings, indicator)
    indicator.isIndeterminate = true
    indicator.text2 = IdeBundle.message("transfersettings.task.progress.details.finishing.up")
    application.invokeAndWait({ performer.performEdt(project, settings) }, indicator.modalityState)

    indicator.text2 = IdeBundle.message("transfersettings.task.progress.details.complete")
  }
}